from typing import List
from typing import Set

from tecton._internals.utils import KEY_DELETION_MAX
from tecton.fco import Fco
from tecton.tecton_errors import TectonInternalError
from tecton.tecton_errors import TectonValidationError

# Generic
INTERNAL_ERROR = lambda message: TectonInternalError(
    f"We seem to have encountered an error. Please contact support for assistance. Error details: {message}"
)
MDS_INACCESSIBLE = lambda host_port: TectonInternalError(
    f"Failed to connect to Tecton cluster at {host_port}, please check your connectivity or contact support"
)
VALIDATION_ERROR_FROM_MDS = lambda message, trace_id: TectonValidationError(f"{message}, trace ID: {trace_id}")
INTERNAL_ERROR_FROM_MDS = lambda message, trace_id: TectonInternalError(
    f"Internal Tecton server error, please contact support with error details: {message}, trace ID: {trace_id}"
)
TIME_KEY_TYPE_ERROR = lambda timestamp_key, found_type: TectonValidationError(
    f"Column '{timestamp_key}' must be Spark type 'TimestampType'. Found '{str(found_type)}'"
)
# Add a link to reference explaining time/duration formats
TIME_PARSING_ERROR = lambda param_name, time_str: TectonValidationError(
    f"Error parsing time '{time_str}' in '{param_name}'"
)
ERROR_RESOLVING_COLUMN = lambda column_name: TectonValidationError(f"Error resolving column '{column_name}'")
FLOATING_POINT_JOIN_KEY = lambda column_name: TectonValidationError(
    f"Floating point types are not allowed for join_key '{column_name}'"
)
INVALID_SPINE_TYPE = lambda t: TectonValidationError(
    f"Invalid type of spine '{t}'. Spine must be an instance of [pyspark.sql.dataframe.DataFrame, " "pandas.DataFrame]."
)
UNSUPPORTED_OPERATION = lambda op, reason: TectonValidationError(f"Operation '{op}' is not supported: {reason}")
INVALID_SPINE_TIME_KEY_TYPE = lambda t: TectonValidationError(
    f"Invalid type of timestamp_key column in the given spine. Expected TimestampType, got {t}"
)
MISSING_SPINE_COLUMN = lambda param, col, existing_cols: TectonValidationError(
    f"TectonValidationError: {param} column is missing from the spine. Expected to find '{col}' among available spine columns: '{', '.join(existing_cols)}'."
)
MISSING_REQUEST_DATA_IN_SPINE = lambda key, existing_cols: TectonValidationError(
    f"TectonValidationError: Request context key '{key}' not found in spine schema. Expected to find '{key}' among available spine columns: '{', '.join(existing_cols)}'."
)
INVALID_DATETIME_PARTITION_COLUMNS = lambda datepart: TectonValidationError(
    f'TectonValidationError: Invalid set of DatetimePartitionColumns: missing "{datepart}"'
)
DURATION_NEGATIVE_OR_AMBIGUOUS = lambda param_name, param_value: TectonValidationError(
    f"{param_name} must be a positive value and must not contain ambiguous durations such as months and years. Instead got {param_value}"
)
NONEXISTENT_WORKSPACE = lambda name, workspaces: TectonValidationError(
    f'Workspace "{name}" not found. Possible values: {workspaces}'
)
INCORRECT_MATERIALIZATION_ENABLED_FLAG = lambda user_set_bool, server_side_bool: TectonValidationError(
    f"'is_live={user_set_bool}' argument does not match the value on the server: {server_side_bool}"
)
UNSUPPORTED_OPERATION_IN_DEVELOPMENT_WORKSPACE = lambda op: TectonValidationError(
    f"Operation '{op}' is not supported in a development workspace"
)
INVALID_JOIN_KEYS_TYPE = lambda t: TectonValidationError(
    f"Invalid type for join_keys. Expected Dict[str, Union[int, str, bytes]], got {t}"
)
INVALID_REQUEST_DATA_TYPE = lambda t: TectonValidationError(
    f"Invalid type for request_data. Expected Dict[str, Union[int, str, bytes, float]], got {t}"
)
INVALID_REQUEST_CONTEXT_TYPE = lambda t: TectonValidationError(
    f"Invalid type for request_context_map. Expected Dict[str, Union[int, str, bytes, float]], got {t}"
)
CONTEXT_REQUIRED = lambda: TectonValidationError("Expected `context` to be a `MaterializationContext` but got `None`.")


def INVALID_INDIVIDUAL_JOIN_KEY_TYPE(key: str, type_str: str):
    return TectonValidationError(
        f"Invalid type for join_key '{key}'. Expected either type int, str, or bytes, got {type_str}"
    )


def EMPTY_ARGUMENT(argument: str):
    return TectonValidationError(f"Argument '{argument}' can not be empty.")


def EMPTY_ELEMENT_IN_ARGUMENT(argument: str):
    return TectonValidationError(f"Argument '{argument}' can not have an empty element.")


def WRONG_ARGUMENT_TYPE(argument, expected_type):
    return TectonValidationError(f"Argument '{argument}' must be of type '{expected_type}'")


def DUPLICATED_ELEMENTS_IN_ARGUMENT(argument: str):
    return TectonValidationError(f"Argument '{argument}' can not have duplicated elements.")


def FCO_NOT_FOUND(fco: Fco, fco_reference: str):
    raise TectonValidationError(
        f"{fco._fco_type_name_singular_capitalized()} '{fco_reference}' not found. "
        f"Try running `tecton.list_{fco._fco_type_name_plural_snake_case()}()` to view all registered "
        f"{fco._fco_type_name_plural_capitalized()}"
    )


def FEATURE_DEFINITION_NOT_FOUND(fco_reference: str):
    raise TectonValidationError(
        f"'{fco_reference}' not found. "
        "Try running tecton.list_feature_views() to view all registered FeatureViews "
        " and tecton.list_feature_tables() to view all registered FeatureTables."
    )


def DATA_SOURCE_NOT_FOUND(fco_reference: str):
    raise TectonValidationError(
        f"Data source '{fco_reference}' not found. "
        f"Try running `tecton.list_data_sources()` to view all registered BatchDataSource/StreamDataSource."
    )


def FCO_NOT_FOUND_WRONG_TYPE(fco: Fco, fco_reference: str, expected_method):
    raise TectonValidationError(
        f"{fco._fco_type_name_singular_capitalized()} '{fco_reference}' not found. "
        f"Did you mean tecton.{expected_method}?"
    )


def UNKNOWN_REQUEST_CONTEXT_KEY(keys, key):
    return TectonValidationError(f"Unknown request context key '{key}', expected one of: {keys}")


FV_TIME_KEY_MISSING = lambda fv_name: TectonValidationError(
    f"Argument 'timestamp_key' is required for the feature definition '{fv_name}'"
)
FV_NEEDS_TO_BE_MATERIALIZED = lambda fv_name: TectonValidationError(
    f"FeatureView '{fv_name}' has not been configured for materialization. "
    + f"Set offline_enabled=True in the FeatureView definition to enable this feature."
)
FV_NO_MATERIALIZED_DATA = lambda fv_name: TectonValidationError(
    f"Feature definition '{fv_name}' doesn't have any materialized data. "
    "Materialization jobs may not have updated the offline feature store yet. "
    "Please monitor using materialization_status() or use from_source=True to compute from source data."
)

FV_NOT_SUPPORTED_PREVIEW = TectonValidationError(
    "This method cannot be used with this type of Feature Definition. Please use get_historical_features(spine=spine)."
)
FD_PREVIEW_NO_MATERIALIZED_OFFLINE_DATA = TectonValidationError(
    f"No materialized offline data found. If this Feature Definition was recently created,"
    + " its materialization backfill may still be in progress. This can be monitored using materialization_status()."
    + " In the meantime, you can set use_materialized_data=False on preview() to compute features directly from data sources."
)
FV_GET_FEATURE_DF_NO_SPINE = TectonValidationError("get_feature_dataframe() requires a 'spine' argument.")
FT_DF_TOO_LARGE = TectonValidationError(
    "Dataframe too large for a single ingestion, consider splitting into smaller ones"
)
FV_BFC_SINGLE_FROM_SOURCE = TectonValidationError(
    f"Computing features from source is not supported for Batch Feature Views with single_batch_schedule_interval_per_job backfill mode"
)

FD_GET_MATERIALIZED_FEATURES_FROM_DEVELOPMENT_WORKSPACE = lambda fd_name, workspace: TectonValidationError(
    f"Feature Definition {fd_name} is in workspace {workspace}, which is a development workspace (does not have materialization enabled). "
    "Please use from_source=True when getting features (not applicable for Feature Tables) or "
    "alternatively configure offline materialization for this Feature Definition in a live workspace."
)

FD_GET_MATERIALIZED_FEATURES_FROM_DEVELOPMENT_WORKSPACE_GFD = lambda fv_name, workspace: TectonValidationError(
    f"Feature View {fv_name} is in workspace {workspace}, which is a development workspace (does not have materialization enabled). "
    "Please use use_materialized_data=False when getting features or "
    "alternatively configure offline materialization for this Feature View in a live workspace."
)


CBFV_GET_MATERIALIZED_FEATURES_FROM_DEVELOPMENT_WORKSPACE = lambda fv_name, workspace: TectonValidationError(
    f"Feature View {fv_name} is in workspace {workspace}, which is a development workspace (does not have materialization enabled). "
    "Please configure offline materialization for this Feature View in a live workspace to call `get_historical_features()`."
)


FD_GET_FEATURES_MATERIALIZATION_DISABLED = lambda fd_name: TectonValidationError(
    f"Feature Definition {fd_name} does not have offline materialization turned on. "
    f"Try calling this function with 'from_source=True' (not applicable for Feature Tables) "
    "or alternatively configure offline materialization for this Feature Definition."
)

CBFV_GET_FEATURES_MATERIALIZATION_DISABLED = lambda fd_name: TectonValidationError(
    f"Feature Definition {fd_name} does not have offline materialization turned on, which is required for `get_historical_features()`. "
    f"Please configure offline materialization for this Feature Definition."
)

FV_GET_FEATURES_MATERIALIZATION_DISABLED_GFD = lambda fv_name: TectonValidationError(
    f"Feature View {fv_name} does not have offline materialization turned on. "
    f"Try calling this function with 'use_materialized_data=False' "
    "or alternatively configure offline materialization for this Feature View."
)


def FT_UPLOAD_FAILED(reason):
    return TectonValidationError(f"Failed to upload dataframe: {reason}")


# DataSources
def DS_STREAM_BATCH_SCHEMA_MISMATCH(columns):
    return TectonValidationError(
        f"Streaming data source schema has column(s) that are not present in the batch data source schema: {columns}"
    )


def DS_INVALID_TABLE_COLUMN(column_name, existing_columns):
    return TectonValidationError(f"Column '{column_name}' is not among table columns {existing_columns}")


def DS_INVALID_TABLE_PARTITION(partition_name, existing_partitions):
    return TectonValidationError(f"Column '{partition_name}' is not among table partitions {existing_partitions}")


def DS_NAME_EQUALS_HIVE_TABLE_NAME(name):
    return TectonValidationError(
        f"DataSource name '{name}' is not allowed because a Glue (or Hive) table with the same name exists. "
        f"This is a current system limitation. Please rename your DataSource to, for instance, '{name}_ds'"
    )


def DS_NAME_CANNOT_CONTAIN_DOT(name):
    return TectonValidationError(f"DataSource name '{name}' cannot contain a dot. Please, remove it and try again.")


def DS_TIMESTAMP_COLUMN_MISSING_FOR_LOOKBACK(name):
    return TectonValidationError(
        "Setting data_lookback requires your upstream data source to specify a timestamp column to enable Tecton to "
        "filter your raw data according to the time range specified by data_lookback. Please specify 'timestamp_column'"
        f" on the batch_ds_config of your DataSource {name} or unset data_lookback to disable Tecton's "
        "raw data filtering."
    )


DS_TIME_KEY_MISSING_ERROR = lambda timestamp_key, all_cols: TectonValidationError(
    f"Timestamp column '{timestamp_key}' missing from Schema. Found columns {all_cols}"
)

DS_STREAM_PREVIEW_ON_NON_STREAM = TectonValidationError("'start_stream_preview' called on non-streaming data source")

DS_DATAFRAME_NO_TIMESTAMP = TectonValidationError(
    "Cannot find timestamp column for this data source. Please call 'get_dataframe' without parameters 'start_time' or 'end_time'."
)

DS_RAW_DATAFRAME_NO_TIMESTAMP_FILTER = TectonValidationError(
    "The method 'get_dataframe()' cannot filter on timestamps when 'apply_translator' is False. "
    "'start_time' and 'end_time' must be None."
)


def DS_DEDUPLICATION_COLUMN_MISSING(column, schema_columns):
    return TectonValidationError(
        f"Deduplication column '{column}' not present in the translated stream schema {schema_columns}"
    )


def FS_SPINE_JOIN_KEY_OVERRIDE_INVALID(spine_key, fv_key, possible_columns):
    return TectonValidationError(
        f"Spine join key '{spine_key}' (mapped from FeatureView join key '{fv_key}') not found in spine schema {possible_columns}"
    )


def FS_SPINE_TIMESTAMP_KEY_INVALID(timestamp_key, possible_columns):
    return TectonValidationError(f"Spine timestamp key '{timestamp_key}' not found in spine schema {possible_columns}")


def FS_BACKEND_ERROR(message):
    return TectonInternalError(f"Error calling Feature Service backend: {message}")


def FS_AMBIGUOUS_TIMESTAMP_KEY(keys):
    return TectonValidationError(f"Multiple default timestamp keys found: {keys}. Parameter timestamp_key must be set.")


FS_GET_FEATURE_VECTOR_REQUIRED_ARGS = TectonValidationError(
    "get_feature_vector requires at least one of join_keys or request_context_map"
)

FS_API_KEY_MISSING = TectonValidationError(
    "API key is required for online feature requests, but was not found in the environment. Please generate a key and set TECTON_API_KEY "
    + "using https://docs.tecton.ai/v2/examples/fetch-real-time-features.html#generating-an-api-key"
)

# Entity
def ENTITY_INVALID_JOIN_KEYS_OVERRIDE(entity_name: str):
    return TectonValidationError(
        f"New join_keys must have the same amount of columns as existing join_keys for entity '{entity_name}'."
    )


def FV_INVALID_MOCK_INPUTS(mock_inputs: Set[str], inputs: Set[str]):
    input_names = str(mock_inputs) if mock_inputs else "{}"
    return TectonValidationError(f"Mock input names {input_names} do not match FeatureView's inputs {inputs}")


def FV_INVALID_MOCK_INPUTS_NUM_ROWS(num_rows: List[int]):
    return TectonValidationError(
        f"Number of rows are not equal across all mock_inputs. Number of rows found are: {str(num_rows)}."
    )


def FV_INVALID_MOCK_INPUT_SCHEMA(input_name: str, mock_columns: Set[str], expected_columns: Set[str]):
    return TectonValidationError(
        f"mock_inputs['{input_name}'] has mismatch schema columns {mock_columns}, expected {expected_columns}"
    )


def FV_UNSUPPORTED_ARG(invalid_arg_name: str):
    return TectonValidationError(f"Argument '{invalid_arg_name}' is not supported for this FeatureView type.")


def FV_INVALID_ARG_VALUE(arg_name: str, value: str, expected: str):
    return TectonValidationError(f"Invalid argument value '{arg_name}={value}', supported value(s): '{expected}'")


def FV_INVALID_ARG_COMBO(arg_names: List[str]):
    return TectonValidationError(f"Invalid argument combinations; {str(arg_names)} cannot be used together.")


FT_UNABLE_TO_ACCESS_SOURCE_DATA = lambda fv_name: TectonValidationError(
    f"The source data for FeatureTable {fv_name} does not exist. "
    "Please use from_source=False when calling this function."
)


class InvalidTransformationMode(TectonValidationError):
    def __init__(self, name: str, got: str, allowed_modes: List[str]):
        super().__init__(f"Transformation mode for '{name}' got '{got}', must be one of: {', '.join(allowed_modes)}")


class InvalidConstantType(TectonValidationError):
    def __init__(self, value, allowed_types):
        allowed_types = [str(allowed_type) for allowed_type in allowed_types]
        super().__init__(
            f"Tecton const value '{value}' must have one of the following types: {', '.join(allowed_types)}"
        )


class InvalidTransformInvocation(TectonValidationError):
    def __init__(self, transformation_name: str, got: str):
        super().__init__(
            f"Allowed arguments for Transformation '{transformation_name}' are: "
            f"tecton.const, tecton.materialization_context, transformations, and DataSource inputs. Got: '{got}'"
        )


# Dataset
DATASET_SPINE_COLUMNS_NOT_SET = TectonValidationError(
    "Cannot retrieve spine DF when DataFrame was created without a " "spine."
)

# Feature Retrevial
def GET_HISTORICAL_FEATURES_WRONG_PARAMS(params: List[str], if_statement: str):
    return TectonValidationError("Cannot provide parameters " + ", ".join(params) + f" if {if_statement}")


FV_NOT_SUPPORTED_GET_HISTORICAL_FEATURES = TectonValidationError(
    "A spine must be provided to perform this method on an On Demand Feature View. Please use get_historical_features(spine=spine)."
)

GET_ONLINE_FEATURES_REQUIRED_ARGS = TectonValidationError(
    "get_online_features requires at least one of join_keys or request_data"
)

GET_ONLINE_FEATURES_ODFV_JOIN_KEYS = TectonValidationError(
    "get_online_features requires the 'join_keys' argument for this Feature View as it has dependent Feature Views as inputs"
)

GET_ONLINE_FEATURES_FS_JOIN_KEYS = TectonValidationError(
    "get_online_features requires the 'join_keys' argument for this Feature Service"
)

GET_FEATURE_VECTOR_FS_JOIN_KEYS = TectonValidationError(
    "get_feature_vector requires the 'join_keys' argument for this Feature Service"
)

FS_GET_ONLINE_FEATURES_REQUIRED_ARGS = TectonValidationError(
    "get_online_features requires at least one of join_keys or request_data"
)


def GET_ONLINE_FEATURES_MISSING_REQUEST_KEY(keys: Set[str]):
    return TectonValidationError(
        f"Missing the following required keys in request_data input: "
        + ", ".join(keys)
        + ". Please provide a value for the keys in request_data."
    )


def GET_FEATURE_VECTOR_MISSING_REQUEST_KEY(keys: Set[str]):
    return TectonValidationError(
        f"Missing the following required keys in request_context_map input: "
        + ", ".join(keys)
        + ". Please provide a value for the keys in request_context_map."
    )


def GET_ONLINE_FEATURES_FS_NO_REQUEST_DATA(keys: List[str]):
    return TectonValidationError(
        "get_online_features requires the 'request_data' argument for this Feature Service. Expected the following request data keys: "
        + ", ".join(keys)
    )


def GET_FEATURE_VECTOR_FS_NO_REQUEST_DATA(keys: List[str]):
    return TectonValidationError(
        "get_feature_vector requires the 'request_context_map' argument for this Feature Service. Expected the following request context keys: "
        + ", ".join(keys)
    )


def GET_ONLINE_FEATURES_FV_NO_REQUEST_DATA(keys: List[str]):
    return TectonValidationError(
        "get_online_features requires the 'request_data' argument for this OnDemand Feature View. Expected the following request data keys: "
        + ", ".join(keys)
    )


def GET_FEATURE_VECTOR_FV_NO_REQUEST_DATA(keys: List[str]):
    return TectonValidationError(
        "get_feature_vector requires the 'request_context_map' argument for this OnDemand Feature View. Expected the following request context keys: "
        + ", ".join(keys)
    )


def GET_ONLINE_FEATURES_MISSING_JOIN_KEYS(missing_key: str):
    return TectonValidationError(f"Join key {missing_key} not found in parameter 'join_keys'")


FROM_SOURCE_WITH_FT = TectonValidationError(
    f"Computing features from source is not supported for Feature Tables. Try calling this method with from_source=False."
)

USE_MATERIALIZED_DATA_WITH_FT = TectonValidationError(
    f"Computing features from source is not supported for Feature Tables. Try calling this method with use_materialized_data=True."
)


FS_WITH_FT_DEVELOPMENT_WORKSPACE = TectonValidationError(
    f"This Feature Service contains a Feature Table and fetching historical features for Feature Tables is not supported in a development workspace. Try calling this method in a live workspace."
)


FV_WITH_FT_DEVELOPMENT_WORKSPACE = TectonValidationError(
    f"This Feature View has a Feature Table input and fetching historical features for Feature Tables is not supported in a development workspace. Try calling this method in a live workspace."
)


# Backfill Config Validation
BFC_MODE_SINGLE_REQUIRED_FEATURE_END_TIME_WHEN_START_TIME_SET = TectonValidationError(
    "feature_end_time is required when feature_start_time is set, for a FeatureView with "
    + "single-batch-schedule-per-job backfill mode."
)

BFC_MODE_SINGLE_INVALID_FEATURE_TIME_RANGE = TectonValidationError(
    "Run with single_batch_schedule_interval_per_job backfill mode only supports time range equal to batch_schedule"
)

INCORRECT_KEYS = lambda keys, join_keys: TectonValidationError(
    f"The requested keys to be deleted ({keys}) do not match the expected keys ({join_keys})."
)

NO_STORE_SELECTED = TectonValidationError(f"One of the online store or offline store needs to be selected.")

TOO_MANY_KEYS = TectonValidationError(f"Max number of keys to be deleted is {KEY_DELETION_MAX}.")

OFFLINE_STORE_NOT_SUPPORTED = TectonValidationError(
    "Only DeltaLake is supported for entity deletion in offline feature stores."
)
ONLINE_STORE_NOT_SUPPORTED = TectonValidationError(
    "Only DynamoDB is supported for entity deletion in online feature stores."
)

# FWv4
FV_UNSUPPORTED_AGGREGATION = TectonValidationError(
    f"Argument 'aggregation_level' is not supported for this FeatureView when it does not have `aggregations` specified."
)
INVALID_KEY_TYPE = lambda t: TectonValidationError(
    f"Invalid type of join keys '{t}'. Keys must be an instance of [pyspark.sql.dataframe.DataFrame, "
    "pandas.DataFrame]."
)
DUPLICATED_COLS_IN_KEYS = lambda t: TectonValidationError(f"Argument keys {t} have duplicated column names. ")



import datetime
from typing import Union

import pendulum
import pytimeparse
from google.protobuf import duration_pb2

from tecton_proto.data.feature_store_pb2 import FeatureStoreFormatVersion
from tecton_spark.errors import TectonValidationError

WINDOW_UNBOUNDED_PRECEDING = "unbounded_preceding"


def timedelta_to_duration(td: datetime.timedelta) -> pendulum.Duration:
    return pendulum.duration(days=td.days, seconds=td.seconds, microseconds=td.microseconds)


def proto_to_duration(proto_duration) -> pendulum.Duration:
    return timedelta_to_duration(proto_duration.ToTimedelta())


def assert_is_round_seconds(d: pendulum.Duration):
    if d % pendulum.Duration(seconds=1):
        raise ValueError(f"{d.in_words()} is not a round number of seconds")
    return d


def assert_valid_time_string(time_str: str, allow_unbounded=False):
    if allow_unbounded:
        if time_str.lower() != WINDOW_UNBOUNDED_PRECEDING and pytimeparse.parse(time_str) is None:
            raise TectonValidationError(
                f"Time string {time_str} must either be `tecton.WINDOW_UNBOUNDED_PRECEDING` or a valid time string."
            )
    else:
        strict_pytimeparse(time_str)


def strict_pytimeparse(time_str: str) -> Union[int, float]:
    parsed = pytimeparse.parse(time_str)
    if parsed is None:
        raise TectonValidationError(f'Could not parse time string "{time_str}"')
    else:
        return parsed


def nanos_to_seconds(nanos: int) -> float:
    """
    :param nanos: Nanoseconds
    :return: Converts nanoseconds to seconds
    """
    return nanos / float(1e9)


def seconds_to_nanos(seconds: int) -> int:
    """
    :param seconds: Seconds
    :return: Converts seconds to nanoseconds
    """
    return int(seconds) * int(1e9)


def convert_timedelta_for_version(duration: datetime.timedelta, version: int) -> int:
    """
    Convert pendulum duration according to version
    VO -> Return Seconds
    V1 -> Return Nanoseconds
    :param duration: Pendulum Duration
    :param version: Feature Store Format Version
    :return:
    """
    assert duration.microseconds == 0
    interval = duration.total_seconds()
    if version == FeatureStoreFormatVersion.FEATURE_STORE_FORMAT_VERSION_DEFAULT:
        return int(interval)
    else:
        return int(seconds_to_nanos(interval))


def convert_proto_duration_for_version(duration: duration_pb2.Duration, version: int) -> int:
    return convert_timedelta_for_version(duration.ToTimedelta(), version)


def align_time_downwards(time: datetime.datetime, alignment: datetime.timedelta) -> datetime.datetime:
    excess_seconds = time.timestamp() % alignment.total_seconds()
    return datetime.datetime.utcfromtimestamp(time.timestamp() - excess_seconds)


def align_time_upwards(time: datetime.datetime, alignment: datetime.timedelta) -> datetime.datetime:
    excess_seconds = time.timestamp() % alignment.total_seconds()
    offset = 0 if excess_seconds == 0 else alignment.total_seconds() - excess_seconds
    return datetime.datetime.utcfromtimestamp(time.timestamp() + offset)



from typing import List
from typing import Optional
from typing import Tuple

import pendulum
from pyspark.sql import functions
from pyspark.sql import SparkSession
from pyspark.sql.window import Window

from tecton._internals import data_frame_helper
from tecton._internals import errors
from tecton_proto.common import aggregation_function_pb2
from tecton_proto.data.feature_store_pb2 import FeatureStoreFormatVersion
from tecton_proto.data.feature_view_pb2 import TrailingTimeWindowAggregation
from tecton_spark import materialization_plan
from tecton_spark import offline_store
from tecton_spark.aggregation_plans import get_aggregation_plan
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.materialization_params import MaterializationParams
from tecton_spark.partial_aggregations import TEMPORAL_ANCHOR_COLUMN_NAME
from tecton_spark.spark_time_utils import convert_epoch_to_datetime
from tecton_spark.spark_time_utils import convert_epoch_to_timestamp_column
from tecton_spark.time_utils import convert_proto_duration_for_version
from tecton_spark.time_utils import convert_timedelta_for_version

"""
This file contains internal methods of feature_views/aggregations.py
Separate file helps de-clutter the main user-visible file.
"""


_TEMPORAL_PREVIEW_DURATION = pendulum.Duration(days=1)


def _get_feature_data_time_limits(fd: FeatureDefinition, spine_time_limits):
    if spine_time_limits is None:
        # If there's no spine, it means we need to preview small portion of the dataframe. So we optimize access from
        # the raw data source by applying artificial time limits to the dataframe
        if fd.is_feature_table:
            time_end = pendulum.now()
            # We choose not to subtract by serving_ttl since it shouldn't be
            # vital for preview functionality and it will improve response
            # times.
            time_start = time_end - _TEMPORAL_PREVIEW_DURATION
            return time_start, time_end
        elif fd.is_temporal or fd.is_temporal_aggregate:
            materialization_params = MaterializationParams.from_feature_definition(fd)
            time_end = materialization_params.most_recent_anchor(pendulum.now("UTC"))
            time_start = time_end
            return time_start, time_end
        elif fd.is_on_demand:
            raise ValueError(f"Invalid invocation on OnDemandFeatureView type for: '{fd.name}'")
        else:
            raise ValueError(f"Unknown feature type for: '{fd.name}'")
    else:
        time_start = spine_time_limits.start
        time_end = spine_time_limits.end

        if fd.is_temporal:
            # Subtract by `serving_ttl` to accommodate for enough feature data for the feature expiration
            serving_ttl = fd.serving_ttl
            time_start = time_start - serving_ttl

        return time_start, time_end


def _align_feature_data_time_limits(
    materialization_params: MaterializationParams, time_start, time_end
) -> Tuple[pendulum.DateTime, pendulum.DateTime]:
    version = materialization_params.feature_definition.get_feature_store_format_version
    time_start = convert_epoch_to_datetime(materialization_params.align_timestamp_left(time_start), version)
    # Since feature data time interval is open on the right, we need to always strictly align right so that
    # with `batch_schedule = 1h`, time end `04:00:00` will be aligned to `05:00:00`.
    time_end = convert_epoch_to_datetime(materialization_params.force_align_timestamp_right(time_end), version)
    return time_start, time_end


def _get_raw_data_time_limits(
    fd: FeatureDefinition, time_start: pendulum.DateTime, time_end: pendulum.DateTime
) -> pendulum.Period:
    if fd.is_temporal_aggregate:
        # Account for final aggregation needing aggregation window prior to earliest timestamp
        max_aggregation_window = fd.max_aggregation_window
        time_start = time_start - max_aggregation_window.ToTimedelta()

    return time_end - time_start


def _get_time_limits(
    fd: FeatureDefinition,
    spine_df,
    spine_time_limits: Optional[pendulum.Period],
):
    """Get the time limits to set on the partially aggregated dataframe."""
    if spine_time_limits is None and spine_df is not None:
        spine_time_limits = data_frame_helper._get_time_limits_of_dataframe(spine_df, fd.timestamp_key)
    time_start, time_end = _get_feature_data_time_limits(fd, spine_time_limits)
    if time_start is None and time_end is None:
        return None

    if not fd.is_feature_table:
        materialization_params = MaterializationParams.from_feature_definition(fd)
        time_start, time_end = _align_feature_data_time_limits(materialization_params, time_start, time_end)

    return _get_raw_data_time_limits(fd, time_start, time_end)


def _get_all_partial_aggregation_tiles_df(
    spark: SparkSession,
    fd: FeatureDefinition,
    use_materialized_data: bool,
    raw_data_time_limits: Optional[pendulum.Period],
    feature_data_time_limits: Optional[pendulum.Period] = None,
):
    """
    Constructs a DF of a partially aggregated data either from materialized or from raw data source with given time limits.
    """

    if not use_materialized_data:
        if feature_data_time_limits is not None:
            _raw_data_time_limits = _get_raw_data_time_limits(
                fd, feature_data_time_limits.start, feature_data_time_limits.end
            )
        else:
            _raw_data_time_limits = raw_data_time_limits

        # TODO(amargvela): For FVs, pass feature data time range here instead of raw data time range
        # cause that's what `get_batch_materialization_plan` expects.
        # This degrades the performance if `window` is large.
        plan = materialization_plan.get_batch_materialization_plan(
            spark=spark,
            feature_definition=fd,
            feature_data_time_limits=_raw_data_time_limits,
            coalesce=10,
            data_sources=fd.data_sources,
            transformations=fd.transformations,
            schedule_interval=fd.batch_materialization_schedule,
        )
        return plan.offline_store_data_frame

    assert fd.materialization_enabled and fd.writes_to_offline_store

    offline_reader_timelimit = raw_data_time_limits
    if feature_data_time_limits is not None:
        offline_reader_timelimit = feature_data_time_limits

    offline_reader = offline_store.get_offline_store_reader(spark, fd)
    return offline_reader.read(offline_reader_timelimit)


def construct_spine_with_all_join_keys_for_TAFV(
    spine_df,
    partial_aggregations_df,
    fd: FeatureDefinition,
    fully_bound_join_keys: List[str],
):
    """
    Appends a spine with wildcard join_key values to result into set of join_keys that should be returned
    by the feature query for BWAFV/SWAFV.

    E.g. Let's say FV has fully bound join_key `A` and wildcard join_key `C`.
    For every row `[a_0, anchor_0]` from the spine, we will have the following rows appended in the
    returned spine:
       [a_0  c_1  anchor_0]
       [a_0  c_2  anchor_0]
        .    .    .
       [a_0  c_k  anchor_0]
    where (`c_1`, ..., `c_k`) represent all the wildcard join_key values such that, the following row is
    present inside `partial_aggregations_df`:
        [a_0, c_i, anchor_i]
    and:
        anchor_0 - max_feature_agg_period < anchor_i <= anchor_0.
    No rows are returned if no `c_i` exists satisfying conditions above.

    :param spine_df: The spine to join against.
    :param partial_aggregations_df: Partially aggregated DataFrame for the FeatureView.
    :param fd: The feature definition
    """
    join_key_conditions = [
        spine_df[join_key] == partial_aggregations_df[join_key] for join_key in fully_bound_join_keys
    ]

    max_aggregation_window = convert_proto_duration_for_version(
        fd.max_aggregation_window, version=fd.get_feature_store_format_version
    )
    join_condition = join_key_conditions + [
        partial_aggregations_df[TEMPORAL_ANCHOR_COLUMN_NAME] <= spine_df[TEMPORAL_ANCHOR_COLUMN_NAME],
        partial_aggregations_df[TEMPORAL_ANCHOR_COLUMN_NAME]
        > spine_df[TEMPORAL_ANCHOR_COLUMN_NAME] - max_aggregation_window,
    ]

    # It is an inner join because, if no wildcard join_key is present in the aggregation window,
    # there should be no features returned for the corresponding set of fully bound join_keys.
    spine_with_all_join_keys = partial_aggregations_df.alias("p").join(spine_df.alias("s"), join_condition, "inner")

    return spine_with_all_join_keys.select(
        [functions.col(f"p.{join_key}").alias(join_key) for join_key in fd.join_keys]
        + [functions.col(f"s.{TEMPORAL_ANCHOR_COLUMN_NAME}").alias(TEMPORAL_ANCHOR_COLUMN_NAME)]
    ).distinct()


def construct_full_tafv_df(
    spark: SparkSession,
    time_aggregation: TrailingTimeWindowAggregation,
    join_keys: List[str],
    feature_store_format_version: FeatureStoreFormatVersion,
    tile_interval: pendulum.Duration,
    fd: FeatureDefinition = None,
    spine_join_keys=None,
    wildcard_join_keys=None,
    spine_df=None,
    all_partial_aggregations_df=None,
    use_materialized_data=True,
    raw_data_time_limits: Optional[pendulum.Period] = None,
    wildcard_key_not_in_spine: bool = False,
):
    """Construct a full time-aggregation data frame from a partial aggregation data frame.

    :param spark: Spark Session
    :param time_aggregation: trailing time window aggregation.
    :param join_keys: join keys to use on the dataframe.
    :param feature_store_format_version: indicates the time precision used by FeatureStore.
    :param tile_interval: Duration of the aggregation tile interval.
    :param fd: Required only if spine_df is provided. The BWAFV/SWAFV object.
    :param spine_df: (Optional) The spine to join against. If present, the returned data frame
        will contain rollups for all (join key, temporal key) combinations that are required
        to compute a full frame from the spine.
    :param all_partial_aggregations_df: (Optional) The full partial
        aggregations data frame to use in place of the a data frame read from the
        materialized parquet tables.
    :param use_materialized_data: (Optional) Use materialized data if materialization is enabled
    :param raw_data_time_limits: (Optional) Spine Time Bounds
    :param wildcard_key_not_in_spine: (Optional) Whether or not the wildcard join_key is present in the spine.
        Defaults to False if spine is not specified or if the FeatureView has no wildcard join_key.
    """
    # TODO: drop anchor time concept from full aggregations. Define start & end times of the aggregation window,
    # use the new concepts for joining, and for returning the temporal aggregate feature dataframes

    if not all_partial_aggregations_df:
        all_partial_aggregations_df = _get_all_partial_aggregation_tiles_df(
            spark, fd, use_materialized_data, raw_data_time_limits
        )
    if wildcard_key_not_in_spine and wildcard_join_keys:
        spine_join_keys.remove(wildcard_join_keys)

    if not spine_df:
        # If spine isn't provided, the fake timestamp equals to anchor time + tile_interval, s.t. the output timestamp
        # completely contains the time range of the fully aggregated window. Note, that ideally we would either subtract
        # 1 second from the timestamp, due to tiles having [start, end) format, or convert tiles in (start, end] format.
        # For now, we're not doing 1 due to it being confusing in preview, and not doing 2 due to it requiring more work
        partial_aggregations_df = all_partial_aggregations_df.withColumn(
            TEMPORAL_ANCHOR_COLUMN_NAME,
            functions.col(TEMPORAL_ANCHOR_COLUMN_NAME)
            + convert_timedelta_for_version(tile_interval, feature_store_format_version),
        )
    else:
        if not fd:
            raise ValueError("fd argument must not be None when spine_df is provided.")
        # TODO: drop anchor column and keep original spine timestamp
        # If spine is provided, round down the time key to the most recent anchor and right join on it + join_keys
        materialization_params = MaterializationParams.from_feature_definition(fd)
        spine_df = spine_df.select(
            materialization_params.most_recent_anchor_time(functions.to_timestamp(time_aggregation.time_key)).alias(
                TEMPORAL_ANCHOR_COLUMN_NAME
            ),
            *spine_join_keys,
        ).distinct()
        # In order to get the training dataframe, we first inner-join all_partial_aggregations_df to spine_df on join keys.
        # This ensures that the resulting df only contains the join keys that we have in spine.
        partial_aggregations_df = all_partial_aggregations_df.join(
            spine_df.select(spine_join_keys).distinct(), spine_join_keys, how="inner"
        )

        if wildcard_key_not_in_spine and wildcard_join_keys:
            # Cache `partial_aggregations_df` as it is used multiple times for joining below
            partial_aggregations_df.cache()

            # If spine does not contain wildcard key, for specific fully bound join_keys and feature timestamp
            # we should find all the values of wildcard join_key that will be present in the feature aggregation
            # window ending at the feature timestamp.
            spine_with_all_join_keys = construct_spine_with_all_join_keys_for_TAFV(
                spine_df,
                partial_aggregations_df,
                fd,
                spine_join_keys,
            )
        else:
            spine_with_all_join_keys = spine_df

        # Make sure the all join keys & anchor times that occur in spine exist in the partially aggregated df
        # This is necessary in the case when the exact anchor time is missing from partially aggregated df,
        # but the previous tiles belonging to the full aggregation window are available. In this case we shouldn't
        # return null, instead we need to aggregate the existing tiles and return the response.
        partial_aggregations_df = partial_aggregations_df.join(
            spine_with_all_join_keys, on=join_keys + [TEMPORAL_ANCHOR_COLUMN_NAME], how="outer"
        )

    aggregations = []
    for feature in time_aggregation.features:
        # We do + 1 since RangeBetween is inclusive, and we do not want to include the last row of the
        # previous tile. See https://github.com/tecton-ai/tecton/pull/1110
        window_duration = pendulum.Duration(seconds=feature.window.ToSeconds())
        upper_range = -(convert_timedelta_for_version(window_duration, feature_store_format_version)) + 1
        window_spec = (
            Window.partitionBy(join_keys)
            .orderBy(functions.col(TEMPORAL_ANCHOR_COLUMN_NAME).asc())
            .rangeBetween(upper_range, 0)
        )
        aggregation_plan = get_aggregation_plan(
            feature.function, feature.function_params, time_aggregation.is_continuous
        )
        names = aggregation_plan.materialized_column_names(feature.input_feature_name)
        input_columns = [functions.col(name) for name in names]

        if feature.function == aggregation_function_pb2.AGGREGATION_FUNCTION_LASTN:
            # There isn't an Scala Encoder that works with a list directly, so instead we wrap the list in an object. Here we
            # strip the object to get just the list.
            aggregations.append(
                aggregation_plan.full_aggregation_transform(names[0], window_spec).values.alias(
                    feature.output_feature_name
                )
            )
        else:
            aggregations.append(
                aggregation_plan.full_aggregation_transform(input_columns, window_spec).alias(
                    feature.output_feature_name
                )
            )

    output_df = partial_aggregations_df.select(join_keys + [TEMPORAL_ANCHOR_COLUMN_NAME] + aggregations)
    if spine_df:
        output_df = output_df.join(spine_df, on=spine_join_keys + [TEMPORAL_ANCHOR_COLUMN_NAME], how="inner")
    output_df = output_df.withColumn(
        TEMPORAL_ANCHOR_COLUMN_NAME,
        convert_epoch_to_timestamp_column(functions.col(TEMPORAL_ANCHOR_COLUMN_NAME), feature_store_format_version),
    )
    return output_df.withColumnRenamed(TEMPORAL_ANCHOR_COLUMN_NAME, time_aggregation.time_key)


def construct_spine_with_all_join_keys_for_TFV(
    spine_df, tile_df, timestamp_column: str, serving_ttl: int, fully_bound_join_keys: List[str], join_keys: List[str]
):
    """
     Appends a spine with wildcard join_key values to result into set of join_keys that should be returned
     by the feature query for BFV/SFV.

     E.g. Let's say FV has fully bound join_key `A` and wildcard join_key `C`.
     For every row `[a_0, feature_ts_0]` from the spine, we will have the following rows appended in the
     returned spine:
        [a_0  c_1  feature_ts_1]
        [a_0  c_2  feature_ts_2]
         .    .    .
        [a_0  c_k  feature_ts_k]
    where (`c_1`, ..., `c_k`) represent all the wildcard join_key values such that, the following row is
    present inside `tile_df`:
        [a_0, c_i, feature_ts_i]
    and:
        feature_ts_0 - serving_ttl < feature_ts_i <= feature_ts_0.

     :param spine_df: The spine to join against.
     :param tile_df: Partially aggregated DataFrame for the FeatureView.
    """

    join_key_conditions = [spine_df[join_key] == tile_df[join_key] for join_key in fully_bound_join_keys]

    join_condition = join_key_conditions + [
        tile_df[timestamp_column] <= spine_df[timestamp_column],
        tile_df[timestamp_column]
        > functions.from_unixtime(functions.unix_timestamp(spine_df[timestamp_column]) - serving_ttl),
    ]

    # It is an inner join because, if no wildcard join_key is present prior to requested timestamp,
    # there should be no features returned for the corresponding set of fully bound join_keys.
    spine_with_all_join_keys = tile_df.alias("t").join(spine_df.alias("s"), join_condition, "inner")

    return spine_with_all_join_keys.select(
        [functions.col(f"t.{join_key}").alias(join_key) for join_key in join_keys]
        + [functions.col(f"s.{timestamp_column}").alias(timestamp_column)]
    ).distinct()


def construct_full_tfv_or_ft_df(
    spark: SparkSession,
    fd: FeatureDefinition,
    spine_df=None,
    all_tiles_df=None,
    use_materialized_data=True,
    raw_data_time_limits: Optional[pendulum.Period] = None,
    wildcard_key_not_in_spine: bool = False,
):
    """Construct a full data frame from a partial aggregation data frame.

    :param spine_df: (Optional) The spine to join against. If present, the returned data frame
        will contain rollups for all join key combinations that are required
        to compute a full frame from the spine.
    :param all_tiles_df: (Optional) The tile data frame to use in place of the a data frame read from the
        materialized parquet tables.
    :param use_materialized_data: (Optional) Use materialized data if materialization is enabled
    :param raw_data_time_limits: (Optional) Spine Time Bounds
    :param wildcard_key_not_in_spine: Whether or not the wildcard join_key is present in the spine.
        Defaults to False if spine is not specified or if the feature definition has no wildcard join_key.
    """
    if not fd.is_temporal and not fd.is_feature_table:
        raise ValueError(f"Invalid invocation on non BFV/SFV/FeatureTable type for: '{fd.name}'")

    timestamp_key = fd.timestamp_key

    # Currently we only support fetching pre-materialized data. In the future we may do
    # fetching from the raw data, or do a hybrid where we materialize dynamically for the missing data
    if not all_tiles_df:
        all_tiles_df = _get_all_partial_aggregation_tiles_df(spark, fd, use_materialized_data, raw_data_time_limits)

    all_join_keys = fd.join_keys + [timestamp_key]
    if not spine_df:
        # For preview we just need to return DataFrame as it is, except we need to drop the unnecessary anchor time
        return all_tiles_df.select(all_join_keys + fd.features)

    wildcard_join_key = fd.wildcard_join_key
    spine_join_keys = fd.join_keys
    if wildcard_key_not_in_spine and wildcard_join_key:
        spine_join_keys.remove(wildcard_join_key)

    # In order to get the training dataframe, we first inner-join all_tiles_df to spine_df on join keys.
    # This ensures that the resulting df only contains the join keys that we have in spine.
    tiles_df = all_tiles_df.join(spine_df.select(spine_join_keys).distinct(), spine_join_keys, how="inner")
    if wildcard_key_not_in_spine and wildcard_join_key:
        # Cache `tiles_df` as it is used multiple times for joining below
        tiles_df.cache()

        # If spine does not contain wildcard key, for specific fully bound join_keys and feature timestamp
        # we should find all the values of wildcard join_key that will be present in the pre-materialized
        # features with a feature timestamp less than feature timestamp from the spine (to avoid data leakeage).
        serving_ttl = int(fd.serving_ttl.total_seconds())
        timestamp_column = fd.timestamp_key
        spine_with_all_join_keys = construct_spine_with_all_join_keys_for_TFV(
            spine_df, tiles_df, timestamp_column, serving_ttl, spine_join_keys, fd.join_keys
        )
    else:
        spine_with_all_join_keys = spine_df.select(all_join_keys).drop_duplicates()

    # Next, we do an outer join of tiles_df & spine_df on join_keys + timestamp columns. This ensures that
    # rows from spine_df are inserted into df with NULL feature values
    # (except if the timestamps exactly match we'll not have NULL feature value).
    tiles_df = tiles_df.join(spine_with_all_join_keys, all_join_keys, how="outer")
    # Next, we run the "last" window function with (-serving_ttl, 0) window while partitioning the data on fv.join_keys.
    # Since the "last" function is run with ignorenulls=True parameter, it'll fill in the feature values of rows originating
    # from spine_df with previous values, which is exactly what we want (fuzzy join on closest earlier event).
    serving_ttl = int(fd.serving_ttl.total_seconds())
    # We want features with timestamp exactly `serving_ttl` ago not to be included, so we pass
    # `-serving_ttl + 1` in a `rangeBetween` clause below.
    window_spec = (
        Window.partitionBy(fd.join_keys)
        .orderBy(functions.col(timestamp_key).cast("long").asc())
        .rangeBetween(-serving_ttl + 1, 0)
    )

    aggregations = []
    for column_name in fd.features:
        aggregations.append(functions.last(column_name, ignorenulls=True).over(window_spec).alias(column_name))

    df = tiles_df.select(all_join_keys + aggregations)
    # Finally, we right-join df with spine_df on fully_bound_join_keys + timestamp, so that we return only the
    # rows that were needed by the user.
    final_joining_keys = spine_join_keys + [timestamp_key]
    df = df.join(spine_df.select(final_joining_keys).distinct(), final_joining_keys, how="right").drop_duplicates()
    return df


def get_all_temporal_ft_features(
    spark: SparkSession,
    fd: FeatureDefinition,
    entities=None,
    use_materialized_data=True,
    feature_data_time_limits: Optional[pendulum.Period] = None,
):
    assert (
        fd.is_temporal or fd.is_feature_table
    ), f"Invalid invocation on non BFV/SFV/FeatureTable type for: '{fd.name}'"

    if not use_materialized_data and fd.is_feature_table:
        raise errors.FT_UNABLE_TO_ACCESS_SOURCE_DATA(fd.name)

    df = _get_all_partial_aggregation_tiles_df(
        spark, fd, use_materialized_data, None, feature_data_time_limits=feature_data_time_limits
    )

    if entities is not None:
        df = df.join(entities.distinct(), on=entities.columns, how="right")

    columns = fd.join_keys + fd.features + [fd.timestamp_key]
    return df.select(*columns)



import datetime
import functools
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

import pendulum
from typeguard import typechecked

from tecton.data_sources.data_source import BatchDataSource
from tecton.data_sources.data_source import StreamDataSource
from tecton.entities.entity import Entity
from tecton.entities.entity import OverriddenEntity
from tecton.feature_views.feature_view import _UnboundedInput
from tecton.feature_views.feature_view import Input
from tecton.feature_views.feature_view import MaterializedFeatureView
from tecton.features_common.feature_configs import DatabricksClusterConfig
from tecton.features_common.feature_configs import DeltaConfig
from tecton.features_common.feature_configs import DynamoConfig
from tecton.features_common.feature_configs import EMRClusterConfig
from tecton.features_common.feature_configs import ExistingClusterConfig
from tecton.features_common.feature_configs import FeatureAggregation
from tecton.features_common.feature_configs import MonitoringConfig
from tecton.features_common.feature_configs import ParquetConfig
from tecton.features_common.feature_configs import RedisConfig
from tecton.transformations.transformation import transformation
from tecton_proto.args.feature_view_pb2 import FeatureViewType
from tecton_proto.args.virtual_data_source_pb2 import DataSourceType

# This is the mode used when the feature view decorator is used on a pipeline function, i.e. one that only contains
# references to transformations and constants.
PIPELINE_MODE = "pipeline"


FRAMEWORK_VERSION_4 = 4


@typechecked
def stream_feature_view(
    *,
    mode: str,
    source: StreamDataSource,
    entities: List[Union[Entity, OverriddenEntity]],
    aggregation_slide_period: Optional[str] = None,  # TAFV ONLY
    aggregations: List[FeatureAggregation] = [],  # TAFV ONLY
    online: Optional[bool] = False,
    offline: Optional[bool] = False,
    ttl: Optional[str] = None,  # TFV ONLY
    feature_start_time: Optional[Union[pendulum.DateTime, datetime.datetime]] = None,
    batch_schedule: Optional[str] = None,
    max_batch_aggregation_interval: Optional[str] = None,
    online_serving_index: Optional[List[str]] = None,
    batch_cluster_config: Optional[Union[ExistingClusterConfig, DatabricksClusterConfig, EMRClusterConfig]] = None,
    stream_cluster_config: Optional[Union[ExistingClusterConfig, DatabricksClusterConfig, EMRClusterConfig]] = None,
    offline_config: Optional[Union[ParquetConfig, DeltaConfig]] = ParquetConfig(),
    online_config: Optional[Union[DynamoConfig, RedisConfig]] = None,
    monitoring: Optional[MonitoringConfig] = None,
    description: str = "",
    owner: str = "",
    family: str = "",
    tags: Optional[Dict[str, str]] = None,
    timestamp_key: Optional[str] = None,
    name_override: Optional[str] = None,
):
    def decorator(user_function):
        if mode == PIPELINE_MODE:
            pipeline_function = user_function
            inferred_transform = None
        else:
            # Separate out the Transformation and manually construct a simple pipeline function.
            # We infer owner/family/tags but not a description.
            inferred_transform = transformation(mode, name_override, description, owner, family, tags)(user_function)

            def pipeline_function(**kwargs):
                return inferred_transform(**kwargs)

        inputs = {source.name: Input(source)}
        nonlocal aggregations
        if aggregations:
            assert aggregation_slide_period, "`aggregation_slide_period` is required if specifying aggregations"
            assert ttl is None, "`ttl` is automatically set for aggregations to the `aggregation_slide_period`"
            feature_view_type = FeatureViewType.FEATURE_VIEW_TYPE_TEMPORAL_AGGREGATE
        else:
            assert (
                aggregation_slide_period is None
            ), "`aggregation_slide_period` can only be specified when using `aggregations`"

            # Explicitly set aggregations to None (in case it is an empty list)
            aggregations = None

            feature_view_type = FeatureViewType.FEATURE_VIEW_TYPE_TEMPORAL

        featureView = MaterializedFeatureView(
            feature_view_type=feature_view_type,
            name=name_override or user_function.__name__,
            pipeline_function=pipeline_function,
            inferred_transform=inferred_transform,
            inputs=inputs,
            entities=entities,
            online=online,
            offline=offline,
            offline_config=offline_config,
            online_config=online_config,
            aggregation_slide_period=aggregation_slide_period,
            aggregations=aggregations,
            ttl=ttl,
            feature_start_time=feature_start_time,
            batch_schedule=batch_schedule,
            max_batch_aggregation_interval=max_batch_aggregation_interval,
            online_serving_index=online_serving_index,
            batch_cluster_config=batch_cluster_config,
            stream_cluster_config=stream_cluster_config,
            monitoring=monitoring,
            description=description,
            owner=owner,
            family=family,
            tags=tags,
            timestamp_key=timestamp_key,
            data_source_type=DataSourceType.STREAM_WITH_BATCH,
            user_function=user_function,
            framework_version=FRAMEWORK_VERSION_4,
            backfill_config=None,
        )
        functools.update_wrapper(featureView, user_function)

        return featureView

    return decorator


@typechecked
def batch_feature_view(
    *,
    mode: str,
    source: BatchDataSource,
    entities: List[Union[Entity, OverriddenEntity]],
    aggregation_slide_period: Optional[str] = None,  # TAFV ONLY
    aggregations: List[FeatureAggregation] = [],  # TAFV ONLY
    online: Optional[bool] = False,
    offline: Optional[bool] = False,
    ttl: Optional[str] = None,  # TFV ONLY
    feature_start_time: Optional[Union[pendulum.DateTime, datetime.datetime]] = None,
    batch_schedule: Optional[str] = None,
    max_batch_aggregation_interval: Optional[str] = None,
    online_serving_index: Optional[List[str]] = None,
    batch_cluster_config: Optional[Union[ExistingClusterConfig, DatabricksClusterConfig, EMRClusterConfig]] = None,
    offline_config: Optional[Union[ParquetConfig, DeltaConfig]] = ParquetConfig(),
    online_config: Optional[Union[DynamoConfig, RedisConfig]] = None,
    monitoring: Optional[MonitoringConfig] = None,
    description: str = "",
    owner: str = "",
    family: str = "",
    tags: Optional[Dict[str, str]] = None,
    timestamp_key: Optional[str] = None,
    name_override: Optional[str] = None,
):
    def decorator(user_function):
        if mode == PIPELINE_MODE:
            pipeline_function = user_function
            inferred_transform = None
        else:
            # Separate out the Transformation and manually construct a simple pipeline function.
            # We infer owner/family/tags but not a description.
            inferred_transform = transformation(mode, name_override, description, owner, family, tags)(user_function)

            def pipeline_function(**kwargs):
                return inferred_transform(**kwargs)

        inputs = {source.name: Input(source)}
        nonlocal aggregations
        if aggregations:
            assert aggregation_slide_period, "`aggregation_slide_period` is required if specifying aggregations"
            assert ttl is None, "`ttl` is automatically set for aggregations to the `aggregation_slide_period`"
            feature_view_type = FeatureViewType.FEATURE_VIEW_TYPE_TEMPORAL_AGGREGATE
        else:
            assert (
                aggregation_slide_period is None
            ), "`aggregation_slide_period` can only be specified when using `aggregations`"

            # Explicitly set aggregations to None (in case it is an empty list)
            aggregations = None

            feature_view_type = FeatureViewType.FEATURE_VIEW_TYPE_TEMPORAL

        featureView = MaterializedFeatureView(
            feature_view_type=feature_view_type,
            name=name_override or user_function.__name__,
            pipeline_function=pipeline_function,
            inferred_transform=inferred_transform,
            inputs=inputs,
            entities=entities,
            online=online,
            offline=offline,
            offline_config=offline_config,
            online_config=online_config,
            aggregation_slide_period=aggregation_slide_period,
            aggregations=aggregations,
            ttl=ttl,
            feature_start_time=feature_start_time,
            batch_schedule=batch_schedule,
            max_batch_aggregation_interval=max_batch_aggregation_interval,
            online_serving_index=online_serving_index,
            batch_cluster_config=batch_cluster_config,
            stream_cluster_config=None,
            monitoring=monitoring,
            description=description,
            owner=owner,
            family=family,
            tags=tags,
            timestamp_key=timestamp_key,
            data_source_type=DataSourceType.BATCH,
            user_function=user_function,
            framework_version=FRAMEWORK_VERSION_4,
            backfill_config=None,
        )
        functools.update_wrapper(featureView, user_function)

        return featureView

    return decorator


@typechecked
def custom_batch_feature_view(
    *,
    mode: str,
    sources: List[BatchDataSource],
    entities: List[Union[Entity, OverriddenEntity]],
    online: Optional[bool] = False,
    offline: Optional[bool] = False,
    ttl: Optional[str] = None,
    feature_start_time: Optional[Union[pendulum.DateTime, datetime.datetime]] = None,
    batch_schedule: Optional[str] = None,
    batch_cluster_config: Optional[Union[ExistingClusterConfig, DatabricksClusterConfig, EMRClusterConfig]] = None,
    offline_config: Optional[Union[ParquetConfig, DeltaConfig]] = ParquetConfig(),
    online_config: Optional[Union[DynamoConfig, RedisConfig]] = None,
    monitoring: Optional[MonitoringConfig] = None,
    description: str = "",
    owner: str = "",
    family: str = "",
    tags: Optional[Dict[str, str]] = None,
    timestamp_key: Optional[str] = None,
    name_override: Optional[str] = None,
):
    def decorator(user_function):
        if mode == PIPELINE_MODE:
            pipeline_function = user_function
            inferred_transform = None
        else:
            # Separate out the Transformation and manually construct a simple pipeline function.
            # We infer owner/family/tags but not a description.
            inferred_transform = transformation(mode, name_override, description, owner, family, tags)(user_function)

            def pipeline_function(**kwargs):
                return inferred_transform(**kwargs)

        inputs = {ds.name: _UnboundedInput(ds) for ds in sources}

        featureView = MaterializedFeatureView(
            feature_view_type=FeatureViewType.FEATURE_VIEW_TYPE_TEMPORAL,
            name=name_override or user_function.__name__,
            pipeline_function=pipeline_function,
            inferred_transform=inferred_transform,
            inputs=inputs,
            entities=entities,
            online=online,
            offline=offline,
            offline_config=offline_config,
            online_config=online_config,
            aggregation_slide_period=None,
            aggregations=None,
            ttl=ttl,
            feature_start_time=feature_start_time,
            batch_schedule=batch_schedule,
            max_batch_aggregation_interval=None,
            online_serving_index=None,
            batch_cluster_config=batch_cluster_config,
            stream_cluster_config=None,
            monitoring=monitoring,
            description=description,
            owner=owner,
            family=family,
            tags=tags,
            timestamp_key=timestamp_key,
            data_source_type=DataSourceType.BATCH,
            user_function=user_function,
            framework_version=FRAMEWORK_VERSION_4,
            is_custom=True,
            backfill_config=None,
        )
        functools.update_wrapper(featureView, user_function)

        return featureView

    return decorator



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/fco_metadata.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/fco_metadata.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$tecton_proto/data/fco_metadata.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ctecton_proto/common/id.proto\"\xbd\x04\n\x0b\x46\x63oMetadata\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x14\n\x05owner\x18\x04 \x01(\tR\x05owner\x12(\n\x10last_modified_by\x18\r \x01(\tR\x0elastModifiedBy\x12\x1f\n\x0bis_archived\x18\x05 \x01(\x08R\nisArchived\x12\x1c\n\tworkspace\x18\x07 \x01(\tR\tworkspace\x12\x45\n\x12workspace_state_id\x18\x0e \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10workspaceStateId\x12\x16\n\x06\x66\x61mily\x18\x08 \x01(\tR\x06\x66\x61mily\x12\x14\n\x05scope\x18\t \x01(\tR\x05scope\x12#\n\rsource_lineno\x18\n \x01(\tR\x0csourceLineno\x12\'\n\x0fsource_filename\x18\x0b \x01(\tR\x0esourceFilename\x12<\n\x04tags\x18\x0c \x03(\x0b\x32(.tecton_proto.data.FcoMetadata.TagsEntryR\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x06\x10\x07\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,])




_FCOMETADATA_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='tecton_proto.data.FcoMetadata.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.data.FcoMetadata.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.data.FcoMetadata.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=635,
  serialized_end=690,
)

_FCOMETADATA = _descriptor.Descriptor(
  name='FcoMetadata',
  full_name='tecton_proto.data.FcoMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.data.FcoMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='tecton_proto.data.FcoMetadata.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='description', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='tecton_proto.data.FcoMetadata.created_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='createdAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='owner', full_name='tecton_proto.data.FcoMetadata.owner', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='owner', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_modified_by', full_name='tecton_proto.data.FcoMetadata.last_modified_by', index=4,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='lastModifiedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_archived', full_name='tecton_proto.data.FcoMetadata.is_archived', index=5,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='isArchived', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.FcoMetadata.workspace', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace_state_id', full_name='tecton_proto.data.FcoMetadata.workspace_state_id', index=7,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspaceStateId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='family', full_name='tecton_proto.data.FcoMetadata.family', index=8,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='family', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scope', full_name='tecton_proto.data.FcoMetadata.scope', index=9,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scope', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_lineno', full_name='tecton_proto.data.FcoMetadata.source_lineno', index=10,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sourceLineno', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_filename', full_name='tecton_proto.data.FcoMetadata.source_filename', index=11,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sourceFilename', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='tecton_proto.data.FcoMetadata.tags', index=12,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tags', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FCOMETADATA_TAGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=123,
  serialized_end=696,
)

_FCOMETADATA_TAGSENTRY.containing_type = _FCOMETADATA
_FCOMETADATA.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FCOMETADATA.fields_by_name['workspace_state_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FCOMETADATA.fields_by_name['tags'].message_type = _FCOMETADATA_TAGSENTRY
DESCRIPTOR.message_types_by_name['FcoMetadata'] = _FCOMETADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FcoMetadata = _reflection.GeneratedProtocolMessageType('FcoMetadata', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _FCOMETADATA_TAGSENTRY,
    '__module__' : 'tecton_proto.data.fco_metadata_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoMetadata.TagsEntry)
    })
  ,
  'DESCRIPTOR' : _FCOMETADATA,
  '__module__' : 'tecton_proto.data.fco_metadata_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoMetadata)
  })
_sym_db.RegisterMessage(FcoMetadata)
_sym_db.RegisterMessage(FcoMetadata.TagsEntry)


DESCRIPTOR._options = None
_FCOMETADATA_TAGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/workspace.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/workspace.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n!tecton_proto/data/workspace.proto\x12\x11tecton_proto.data\"?\n\x15WorkspaceCapabilities\x12&\n\x0ematerializable\x18\x01 \x01(\x08R\x0ematerializable\"m\n\tWorkspace\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12L\n\x0c\x63\x61pabilities\x18\x02 \x01(\x0b\x32(.tecton_proto.data.WorkspaceCapabilitiesR\x0c\x63\x61pabilitiesB\x13\n\x0f\x63om.tecton.dataP\x01'
)




_WORKSPACECAPABILITIES = _descriptor.Descriptor(
  name='WorkspaceCapabilities',
  full_name='tecton_proto.data.WorkspaceCapabilities',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='materializable', full_name='tecton_proto.data.WorkspaceCapabilities.materializable', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='materializable', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=119,
)


_WORKSPACE = _descriptor.Descriptor(
  name='Workspace',
  full_name='tecton_proto.data.Workspace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.data.Workspace.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='capabilities', full_name='tecton_proto.data.Workspace.capabilities', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='capabilities', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=230,
)

_WORKSPACE.fields_by_name['capabilities'].message_type = _WORKSPACECAPABILITIES
DESCRIPTOR.message_types_by_name['WorkspaceCapabilities'] = _WORKSPACECAPABILITIES
DESCRIPTOR.message_types_by_name['Workspace'] = _WORKSPACE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

WorkspaceCapabilities = _reflection.GeneratedProtocolMessageType('WorkspaceCapabilities', (_message.Message,), {
  'DESCRIPTOR' : _WORKSPACECAPABILITIES,
  '__module__' : 'tecton_proto.data.workspace_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.WorkspaceCapabilities)
  })
_sym_db.RegisterMessage(WorkspaceCapabilities)

Workspace = _reflection.GeneratedProtocolMessageType('Workspace', (_message.Message,), {
  'DESCRIPTOR' : _WORKSPACE,
  '__module__' : 'tecton_proto.data.workspace_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.Workspace)
  })
_sym_db.RegisterMessage(Workspace)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/data_source_config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/data_source_config.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n*tecton_proto/args/data_source_config.proto\x12\x11tecton_proto.args\x1a\x1ctecton_proto/common/id.proto\x1a\x1egoogle/protobuf/duration.proto\"\x13\n\x0b\x42\x61tchConfigJ\x04\x08\x01\x10\x02\"\xc7\x01\n\x0cStreamConfig\x12`\n\x17initial_stream_position\x18\x01 \x01(\x0e\x32(.tecton_proto.args.InitialStreamPositionR\x15initialStreamPosition\x12U\n\x19watermark_delay_threshold\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x17watermarkDelayThreshold*\x94\x01\n\x15InitialStreamPosition\x12\'\n#INITIAL_STREAM_POSITION_UNSPECIFIED\x10\x00\x12\"\n\x1eINITIAL_STREAM_POSITION_LATEST\x10\x01\x12(\n$INITIAL_STREAM_POSITION_TRIM_HORIZON\x10\x03\"\x04\x08\x02\x10\x02\x42\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,])

_INITIALSTREAMPOSITION = _descriptor.EnumDescriptor(
  name='InitialStreamPosition',
  full_name='tecton_proto.args.InitialStreamPosition',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INITIAL_STREAM_POSITION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INITIAL_STREAM_POSITION_LATEST', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INITIAL_STREAM_POSITION_TRIM_HORIZON', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=351,
  serialized_end=499,
)
_sym_db.RegisterEnumDescriptor(_INITIALSTREAMPOSITION)

InitialStreamPosition = enum_type_wrapper.EnumTypeWrapper(_INITIALSTREAMPOSITION)
INITIAL_STREAM_POSITION_UNSPECIFIED = 0
INITIAL_STREAM_POSITION_LATEST = 1
INITIAL_STREAM_POSITION_TRIM_HORIZON = 3



_BATCHCONFIG = _descriptor.Descriptor(
  name='BatchConfig',
  full_name='tecton_proto.args.BatchConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=146,
)


_STREAMCONFIG = _descriptor.Descriptor(
  name='StreamConfig',
  full_name='tecton_proto.args.StreamConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_stream_position', full_name='tecton_proto.args.StreamConfig.initial_stream_position', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='initialStreamPosition', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='watermark_delay_threshold', full_name='tecton_proto.args.StreamConfig.watermark_delay_threshold', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='watermarkDelayThreshold', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=348,
)

_STREAMCONFIG.fields_by_name['initial_stream_position'].enum_type = _INITIALSTREAMPOSITION
_STREAMCONFIG.fields_by_name['watermark_delay_threshold'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['BatchConfig'] = _BATCHCONFIG
DESCRIPTOR.message_types_by_name['StreamConfig'] = _STREAMCONFIG
DESCRIPTOR.enum_types_by_name['InitialStreamPosition'] = _INITIALSTREAMPOSITION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BatchConfig = _reflection.GeneratedProtocolMessageType('BatchConfig', (_message.Message,), {
  'DESCRIPTOR' : _BATCHCONFIG,
  '__module__' : 'tecton_proto.args.data_source_config_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.BatchConfig)
  })
_sym_db.RegisterMessage(BatchConfig)

StreamConfig = _reflection.GeneratedProtocolMessageType('StreamConfig', (_message.Message,), {
  'DESCRIPTOR' : _STREAMCONFIG,
  '__module__' : 'tecton_proto.args.data_source_config_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.StreamConfig)
  })
_sym_db.RegisterMessage(StreamConfig)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/pipeline.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2
from tecton_proto.args import feature_service_pb2 as tecton__proto_dot_args_dot_feature__service__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/pipeline.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n tecton_proto/args/pipeline.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1ctecton_proto/common/id.proto\x1a&tecton_proto/common/spark_schema.proto\x1a\'tecton_proto/args/feature_service.proto\x1a$tecton_proto/args/diff_options.proto\"\xc6\x01\n\x08Pipeline\x12\x33\n\x04root\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04root\x12\x33\n\x04mode\x18\x02 \x01(\x0e\x32\x1f.tecton_proto.args.PipelineModeR\x04mode\x12P\n\x11resolved_pipeline\x18\x03 \x01(\x0b\x32#.tecton_proto.args.ResolvedPipelineR\x10resolvedPipeline\"H\n\x1cSnowflakeSQLResolvedPipeline\x12(\n\x10root_inlined_sql\x18\x01 \x01(\tR\x0erootInlinedSql\"r\n\x10ResolvedPipeline\x12V\n\rsnowflake_sql\x18\x01 \x01(\x0b\x32/.tecton_proto.args.SnowflakeSQLResolvedPipelineH\x00R\x0csnowflakeSqlB\x06\n\x04type\"\xbc\x04\n\x0cPipelineNode\x12X\n\x13transformation_node\x18\x01 \x01(\x0b\x32%.tecton_proto.args.TransformationNodeH\x00R\x12transformationNode\x12M\n\x10\x64\x61ta_source_node\x18\x02 \x01(\x0b\x32!.tecton_proto.args.DataSourceNodeH\x00R\x0e\x64\x61taSourceNode\x12\x46\n\rconstant_node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.ConstantNodeH\x00R\x0c\x63onstantNode\x12\x63\n\x18request_data_source_node\x18\x04 \x01(\x0b\x32(.tecton_proto.args.RequestDataSourceNodeH\x00R\x15requestDataSourceNode\x12P\n\x11\x66\x65\x61ture_view_node\x18\x05 \x01(\x0b\x32\".tecton_proto.args.FeatureViewNodeH\x00R\x0f\x66\x65\x61tureViewNode\x12q\n\x1cmaterialization_context_node\x18\x07 \x01(\x0b\x32-.tecton_proto.args.MaterializationContextNodeH\x00R\x1amaterializationContextNodeB\x0b\n\tnode_typeJ\x04\x08\x06\x10\x07\"\x89\x01\n\x15RequestDataSourceNode\x12J\n\x0frequest_context\x18\x01 \x01(\x0b\x32!.tecton_proto.args.RequestContextR\x0erequestContext\x12$\n\ninput_name\x18\x02 \x01(\tB\x05\x92M\x02\x08\x03R\tinputName\"J\n\x0eRequestContext\x12\x38\n\x06schema\x18\x01 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x06schema\"\xcc\x01\n\x0f\x46\x65\x61tureViewNode\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12R\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\x0b\x32/.tecton_proto.args.FeatureServiceFeaturePackageR\x0b\x66\x65\x61tureView\x12$\n\ninput_name\x18\x03 \x01(\tB\x05\x92M\x02\x08\x03R\tinputName\"\x8c\x01\n\x12TransformationNode\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x30\n\x06inputs\x18\x02 \x03(\x0b\x32\x18.tecton_proto.args.InputR\x06inputs\"\x84\x01\n\x05Input\x12\x1d\n\targ_index\x18\x01 \x01(\x05H\x00R\x08\x61rgIndex\x12\x1b\n\x08\x61rg_name\x18\x02 \x01(\tH\x00R\x07\x61rgName\x12\x33\n\x04node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04nodeB\n\n\x08\x61rg_type\"\x82\x03\n\x0e\x44\x61taSourceNode\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x33\n\x06window\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x06window\x12>\n\x1awindow_unbounded_preceding\x18\x04 \x01(\x08H\x00R\x18windowUnboundedPreceding\x12+\n\x10window_unbounded\x18\x06 \x01(\x08H\x00R\x0fwindowUnbounded\x12\x42\n\x0fschedule_offset\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0escheduleOffset\x12$\n\ninput_name\x18\x05 \x01(\tB\x05\x92M\x02\x08\x03R\tinputNameB\x16\n\x14\x64\x61ta_window_override\"\xe0\x01\n\x0c\x43onstantNode\x12#\n\x0cstring_const\x18\x01 \x01(\tH\x00R\x0bstringConst\x12\x1d\n\tint_const\x18\x02 \x01(\tH\x00R\x08intConst\x12!\n\x0b\x66loat_const\x18\x03 \x01(\tH\x00R\nfloatConst\x12\x1f\n\nbool_const\x18\x04 \x01(\x08H\x00R\tboolConst\x12\x37\n\nnull_const\x18\x05 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\tnullConstB\x0f\n\rconstant_type\"\x1c\n\x1aMaterializationContextNode*\x80\x01\n\x0cPipelineMode\x12\x19\n\x15PIPELINE_MODE_UNKNOWN\x10\x00\x12\x17\n\x13PIPELINE_MODE_SPARK\x10\x01\x12\x1b\n\x17PIPELINE_MODE_ON_DEMAND\x10\x02\x12\x1f\n\x1bPIPELINE_MODE_SNOWFLAKE_SQL\x10\x03\x42\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_spark__schema__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_feature__service__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_diff__options__pb2.DESCRIPTOR,])

_PIPELINEMODE = _descriptor.EnumDescriptor(
  name='PipelineMode',
  full_name='tecton_proto.args.PipelineMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_MODE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_MODE_SPARK', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_MODE_ON_DEMAND', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_MODE_SNOWFLAKE_SQL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2579,
  serialized_end=2707,
)
_sym_db.RegisterEnumDescriptor(_PIPELINEMODE)

PipelineMode = enum_type_wrapper.EnumTypeWrapper(_PIPELINEMODE)
PIPELINE_MODE_UNKNOWN = 0
PIPELINE_MODE_SPARK = 1
PIPELINE_MODE_ON_DEMAND = 2
PIPELINE_MODE_SNOWFLAKE_SQL = 3



_PIPELINE = _descriptor.Descriptor(
  name='Pipeline',
  full_name='tecton_proto.args.Pipeline',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='root', full_name='tecton_proto.args.Pipeline.root', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='root', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='tecton_proto.args.Pipeline.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='mode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resolved_pipeline', full_name='tecton_proto.args.Pipeline.resolved_pipeline', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resolvedPipeline', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=266,
  serialized_end=464,
)


_SNOWFLAKESQLRESOLVEDPIPELINE = _descriptor.Descriptor(
  name='SnowflakeSQLResolvedPipeline',
  full_name='tecton_proto.args.SnowflakeSQLResolvedPipeline',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='root_inlined_sql', full_name='tecton_proto.args.SnowflakeSQLResolvedPipeline.root_inlined_sql', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rootInlinedSql', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=466,
  serialized_end=538,
)


_RESOLVEDPIPELINE = _descriptor.Descriptor(
  name='ResolvedPipeline',
  full_name='tecton_proto.args.ResolvedPipeline',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='snowflake_sql', full_name='tecton_proto.args.ResolvedPipeline.snowflake_sql', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='snowflakeSql', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='type', full_name='tecton_proto.args.ResolvedPipeline.type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=540,
  serialized_end=654,
)


_PIPELINENODE = _descriptor.Descriptor(
  name='PipelineNode',
  full_name='tecton_proto.args.PipelineNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transformation_node', full_name='tecton_proto.args.PipelineNode.transformation_node', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformationNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_source_node', full_name='tecton_proto.args.PipelineNode.data_source_node', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataSourceNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constant_node', full_name='tecton_proto.args.PipelineNode.constant_node', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='constantNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_data_source_node', full_name='tecton_proto.args.PipelineNode.request_data_source_node', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='requestDataSourceNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_node', full_name='tecton_proto.args.PipelineNode.feature_view_node', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='materialization_context_node', full_name='tecton_proto.args.PipelineNode.materialization_context_node', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='materializationContextNode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='node_type', full_name='tecton_proto.args.PipelineNode.node_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=657,
  serialized_end=1229,
)


_REQUESTDATASOURCENODE = _descriptor.Descriptor(
  name='RequestDataSourceNode',
  full_name='tecton_proto.args.RequestDataSourceNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_context', full_name='tecton_proto.args.RequestDataSourceNode.request_context', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='requestContext', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_name', full_name='tecton_proto.args.RequestDataSourceNode.input_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\003', json_name='inputName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1232,
  serialized_end=1369,
)


_REQUESTCONTEXT = _descriptor.Descriptor(
  name='RequestContext',
  full_name='tecton_proto.args.RequestContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='schema', full_name='tecton_proto.args.RequestContext.schema', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1371,
  serialized_end=1445,
)


_FEATUREVIEWNODE = _descriptor.Descriptor(
  name='FeatureViewNode',
  full_name='tecton_proto.args.FeatureViewNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view_id', full_name='tecton_proto.args.FeatureViewNode.feature_view_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='tecton_proto.args.FeatureViewNode.feature_view', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureView', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_name', full_name='tecton_proto.args.FeatureViewNode.input_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\003', json_name='inputName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1448,
  serialized_end=1652,
)


_TRANSFORMATIONNODE = _descriptor.Descriptor(
  name='TransformationNode',
  full_name='tecton_proto.args.TransformationNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transformation_id', full_name='tecton_proto.args.TransformationNode.transformation_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformationId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='tecton_proto.args.TransformationNode.inputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inputs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1655,
  serialized_end=1795,
)


_INPUT = _descriptor.Descriptor(
  name='Input',
  full_name='tecton_proto.args.Input',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='arg_index', full_name='tecton_proto.args.Input.arg_index', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='argIndex', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arg_name', full_name='tecton_proto.args.Input.arg_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='argName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='node', full_name='tecton_proto.args.Input.node', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='node', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='arg_type', full_name='tecton_proto.args.Input.arg_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1798,
  serialized_end=1930,
)


_DATASOURCENODE = _descriptor.Descriptor(
  name='DataSourceNode',
  full_name='tecton_proto.args.DataSourceNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_source_id', full_name='tecton_proto.args.DataSourceNode.virtual_data_source_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSourceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window', full_name='tecton_proto.args.DataSourceNode.window', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='window', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window_unbounded_preceding', full_name='tecton_proto.args.DataSourceNode.window_unbounded_preceding', index=2,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='windowUnboundedPreceding', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window_unbounded', full_name='tecton_proto.args.DataSourceNode.window_unbounded', index=3,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='windowUnbounded', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schedule_offset', full_name='tecton_proto.args.DataSourceNode.schedule_offset', index=4,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scheduleOffset', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_name', full_name='tecton_proto.args.DataSourceNode.input_name', index=5,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\003', json_name='inputName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data_window_override', full_name='tecton_proto.args.DataSourceNode.data_window_override',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1933,
  serialized_end=2319,
)


_CONSTANTNODE = _descriptor.Descriptor(
  name='ConstantNode',
  full_name='tecton_proto.args.ConstantNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_const', full_name='tecton_proto.args.ConstantNode.string_const', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='stringConst', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int_const', full_name='tecton_proto.args.ConstantNode.int_const', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intConst', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_const', full_name='tecton_proto.args.ConstantNode.float_const', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='floatConst', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_const', full_name='tecton_proto.args.ConstantNode.bool_const', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='boolConst', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='null_const', full_name='tecton_proto.args.ConstantNode.null_const', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nullConst', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='constant_type', full_name='tecton_proto.args.ConstantNode.constant_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2322,
  serialized_end=2546,
)


_MATERIALIZATIONCONTEXTNODE = _descriptor.Descriptor(
  name='MaterializationContextNode',
  full_name='tecton_proto.args.MaterializationContextNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2548,
  serialized_end=2576,
)

_PIPELINE.fields_by_name['root'].message_type = _PIPELINENODE
_PIPELINE.fields_by_name['mode'].enum_type = _PIPELINEMODE
_PIPELINE.fields_by_name['resolved_pipeline'].message_type = _RESOLVEDPIPELINE
_RESOLVEDPIPELINE.fields_by_name['snowflake_sql'].message_type = _SNOWFLAKESQLRESOLVEDPIPELINE
_RESOLVEDPIPELINE.oneofs_by_name['type'].fields.append(
  _RESOLVEDPIPELINE.fields_by_name['snowflake_sql'])
_RESOLVEDPIPELINE.fields_by_name['snowflake_sql'].containing_oneof = _RESOLVEDPIPELINE.oneofs_by_name['type']
_PIPELINENODE.fields_by_name['transformation_node'].message_type = _TRANSFORMATIONNODE
_PIPELINENODE.fields_by_name['data_source_node'].message_type = _DATASOURCENODE
_PIPELINENODE.fields_by_name['constant_node'].message_type = _CONSTANTNODE
_PIPELINENODE.fields_by_name['request_data_source_node'].message_type = _REQUESTDATASOURCENODE
_PIPELINENODE.fields_by_name['feature_view_node'].message_type = _FEATUREVIEWNODE
_PIPELINENODE.fields_by_name['materialization_context_node'].message_type = _MATERIALIZATIONCONTEXTNODE
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['transformation_node'])
_PIPELINENODE.fields_by_name['transformation_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['data_source_node'])
_PIPELINENODE.fields_by_name['data_source_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['constant_node'])
_PIPELINENODE.fields_by_name['constant_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['request_data_source_node'])
_PIPELINENODE.fields_by_name['request_data_source_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['feature_view_node'])
_PIPELINENODE.fields_by_name['feature_view_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_PIPELINENODE.oneofs_by_name['node_type'].fields.append(
  _PIPELINENODE.fields_by_name['materialization_context_node'])
_PIPELINENODE.fields_by_name['materialization_context_node'].containing_oneof = _PIPELINENODE.oneofs_by_name['node_type']
_REQUESTDATASOURCENODE.fields_by_name['request_context'].message_type = _REQUESTCONTEXT
_REQUESTCONTEXT.fields_by_name['schema'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_FEATUREVIEWNODE.fields_by_name['feature_view_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FEATUREVIEWNODE.fields_by_name['feature_view'].message_type = tecton__proto_dot_args_dot_feature__service__pb2._FEATURESERVICEFEATUREPACKAGE
_TRANSFORMATIONNODE.fields_by_name['transformation_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_TRANSFORMATIONNODE.fields_by_name['inputs'].message_type = _INPUT
_INPUT.fields_by_name['node'].message_type = _PIPELINENODE
_INPUT.oneofs_by_name['arg_type'].fields.append(
  _INPUT.fields_by_name['arg_index'])
_INPUT.fields_by_name['arg_index'].containing_oneof = _INPUT.oneofs_by_name['arg_type']
_INPUT.oneofs_by_name['arg_type'].fields.append(
  _INPUT.fields_by_name['arg_name'])
_INPUT.fields_by_name['arg_name'].containing_oneof = _INPUT.oneofs_by_name['arg_type']
_DATASOURCENODE.fields_by_name['virtual_data_source_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_DATASOURCENODE.fields_by_name['window'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_DATASOURCENODE.fields_by_name['schedule_offset'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_DATASOURCENODE.oneofs_by_name['data_window_override'].fields.append(
  _DATASOURCENODE.fields_by_name['window'])
_DATASOURCENODE.fields_by_name['window'].containing_oneof = _DATASOURCENODE.oneofs_by_name['data_window_override']
_DATASOURCENODE.oneofs_by_name['data_window_override'].fields.append(
  _DATASOURCENODE.fields_by_name['window_unbounded_preceding'])
_DATASOURCENODE.fields_by_name['window_unbounded_preceding'].containing_oneof = _DATASOURCENODE.oneofs_by_name['data_window_override']
_DATASOURCENODE.oneofs_by_name['data_window_override'].fields.append(
  _DATASOURCENODE.fields_by_name['window_unbounded'])
_DATASOURCENODE.fields_by_name['window_unbounded'].containing_oneof = _DATASOURCENODE.oneofs_by_name['data_window_override']
_CONSTANTNODE.fields_by_name['null_const'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_CONSTANTNODE.oneofs_by_name['constant_type'].fields.append(
  _CONSTANTNODE.fields_by_name['string_const'])
_CONSTANTNODE.fields_by_name['string_const'].containing_oneof = _CONSTANTNODE.oneofs_by_name['constant_type']
_CONSTANTNODE.oneofs_by_name['constant_type'].fields.append(
  _CONSTANTNODE.fields_by_name['int_const'])
_CONSTANTNODE.fields_by_name['int_const'].containing_oneof = _CONSTANTNODE.oneofs_by_name['constant_type']
_CONSTANTNODE.oneofs_by_name['constant_type'].fields.append(
  _CONSTANTNODE.fields_by_name['float_const'])
_CONSTANTNODE.fields_by_name['float_const'].containing_oneof = _CONSTANTNODE.oneofs_by_name['constant_type']
_CONSTANTNODE.oneofs_by_name['constant_type'].fields.append(
  _CONSTANTNODE.fields_by_name['bool_const'])
_CONSTANTNODE.fields_by_name['bool_const'].containing_oneof = _CONSTANTNODE.oneofs_by_name['constant_type']
_CONSTANTNODE.oneofs_by_name['constant_type'].fields.append(
  _CONSTANTNODE.fields_by_name['null_const'])
_CONSTANTNODE.fields_by_name['null_const'].containing_oneof = _CONSTANTNODE.oneofs_by_name['constant_type']
DESCRIPTOR.message_types_by_name['Pipeline'] = _PIPELINE
DESCRIPTOR.message_types_by_name['SnowflakeSQLResolvedPipeline'] = _SNOWFLAKESQLRESOLVEDPIPELINE
DESCRIPTOR.message_types_by_name['ResolvedPipeline'] = _RESOLVEDPIPELINE
DESCRIPTOR.message_types_by_name['PipelineNode'] = _PIPELINENODE
DESCRIPTOR.message_types_by_name['RequestDataSourceNode'] = _REQUESTDATASOURCENODE
DESCRIPTOR.message_types_by_name['RequestContext'] = _REQUESTCONTEXT
DESCRIPTOR.message_types_by_name['FeatureViewNode'] = _FEATUREVIEWNODE
DESCRIPTOR.message_types_by_name['TransformationNode'] = _TRANSFORMATIONNODE
DESCRIPTOR.message_types_by_name['Input'] = _INPUT
DESCRIPTOR.message_types_by_name['DataSourceNode'] = _DATASOURCENODE
DESCRIPTOR.message_types_by_name['ConstantNode'] = _CONSTANTNODE
DESCRIPTOR.message_types_by_name['MaterializationContextNode'] = _MATERIALIZATIONCONTEXTNODE
DESCRIPTOR.enum_types_by_name['PipelineMode'] = _PIPELINEMODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Pipeline = _reflection.GeneratedProtocolMessageType('Pipeline', (_message.Message,), {
  'DESCRIPTOR' : _PIPELINE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.Pipeline)
  })
_sym_db.RegisterMessage(Pipeline)

SnowflakeSQLResolvedPipeline = _reflection.GeneratedProtocolMessageType('SnowflakeSQLResolvedPipeline', (_message.Message,), {
  'DESCRIPTOR' : _SNOWFLAKESQLRESOLVEDPIPELINE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.SnowflakeSQLResolvedPipeline)
  })
_sym_db.RegisterMessage(SnowflakeSQLResolvedPipeline)

ResolvedPipeline = _reflection.GeneratedProtocolMessageType('ResolvedPipeline', (_message.Message,), {
  'DESCRIPTOR' : _RESOLVEDPIPELINE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.ResolvedPipeline)
  })
_sym_db.RegisterMessage(ResolvedPipeline)

PipelineNode = _reflection.GeneratedProtocolMessageType('PipelineNode', (_message.Message,), {
  'DESCRIPTOR' : _PIPELINENODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.PipelineNode)
  })
_sym_db.RegisterMessage(PipelineNode)

RequestDataSourceNode = _reflection.GeneratedProtocolMessageType('RequestDataSourceNode', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTDATASOURCENODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.RequestDataSourceNode)
  })
_sym_db.RegisterMessage(RequestDataSourceNode)

RequestContext = _reflection.GeneratedProtocolMessageType('RequestContext', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTCONTEXT,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.RequestContext)
  })
_sym_db.RegisterMessage(RequestContext)

FeatureViewNode = _reflection.GeneratedProtocolMessageType('FeatureViewNode', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREVIEWNODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.FeatureViewNode)
  })
_sym_db.RegisterMessage(FeatureViewNode)

TransformationNode = _reflection.GeneratedProtocolMessageType('TransformationNode', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMATIONNODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.TransformationNode)
  })
_sym_db.RegisterMessage(TransformationNode)

Input = _reflection.GeneratedProtocolMessageType('Input', (_message.Message,), {
  'DESCRIPTOR' : _INPUT,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.Input)
  })
_sym_db.RegisterMessage(Input)

DataSourceNode = _reflection.GeneratedProtocolMessageType('DataSourceNode', (_message.Message,), {
  'DESCRIPTOR' : _DATASOURCENODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.DataSourceNode)
  })
_sym_db.RegisterMessage(DataSourceNode)

ConstantNode = _reflection.GeneratedProtocolMessageType('ConstantNode', (_message.Message,), {
  'DESCRIPTOR' : _CONSTANTNODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.ConstantNode)
  })
_sym_db.RegisterMessage(ConstantNode)

MaterializationContextNode = _reflection.GeneratedProtocolMessageType('MaterializationContextNode', (_message.Message,), {
  'DESCRIPTOR' : _MATERIALIZATIONCONTEXTNODE,
  '__module__' : 'tecton_proto.args.pipeline_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.MaterializationContextNode)
  })
_sym_db.RegisterMessage(MaterializationContextNode)


DESCRIPTOR._options = None
_REQUESTDATASOURCENODE.fields_by_name['input_name']._options = None
_FEATUREVIEWNODE.fields_by_name['input_name']._options = None
_DATASOURCENODE.fields_by_name['input_name']._options = None
# @@protoc_insertion_point(module_scope)
